<?xml version="1.0" encoding="UTF-8"?>

<xsl:stylesheet version="1.0" xmlns:xsl="http://www.w3.org/1999/XSL/Transform" xmlns:g="http://www.interactive-instruments.de/namespaces/GdbDoc" xmlns:xlink="http://www.w3.org/1999/xlink">
	<xsl:output method="html"/>
	<xsl:template match="/">
  	<html>
			<head>
				<xsl:choose>
					<xsl:when test="/g:Geodatabase">
						<title>Geodatabase '<xsl:value-of select="/g:Geodatabase/@name"/>'</title>
					</xsl:when>
					<xsl:when test="/g:Type">
						<title><xsl:value-of select="/g:Type/@type"/> '<xsl:value-of select="/g:Type/@name"/>'</title>
					</xsl:when>
				</xsl:choose>
				<style type="text/css">
				body
				{
				background-color:#f4f6fe;
				}
				h1
				{
				font-family:Arial, Helvetica, sans-serif;
				font-size:24px;
				color:#151B8D;
				text-align:center;
				}
				h2, h3, h4
				{
				font-family:Arial, Helvetica, sans-serif;
				color:#151B8D;
				}
				p, li
				{
				font-family:Arial, Helvetica, sans-serif;
				font-size:12px;
				}
				.rightalign
				{
				font-size:10px;
				text-align:right;
				}
				.small
				{
				font-size:10px;
				text-align:center;
				}
				#subtypes
				{
				font-family:Arial, Helvetica, sans-serif;
				width:100%;
				border-collapse:collapse;
				}
				#subtypes tr, #fields tr 
				{
				vertical-align:top;
				}
				#subtypes td, #subtypes th 
				{
				font-size:1.2em;
				border:1px solid #98bf21;
				padding:3px 7px 2px 7px;
				}
				#subtypes th 
				{
				font-size:1.4em;
				text-align:left;
				padding-top:5px;
				padding-bottom:4px;
				background-color:#A7C942;
				color:#fff;
				}
				#subtypes tr.alt td 
				{
				color:#000;
				background-color:#EAF2D3;
				}
				#fields 
				{
				font-family:Arial, Helvetica, sans-serif;
				width:100%;
				border-collapse:collapse;
				}
				#fields td, #fields th 
				{
				font-size:1.2em;
				border:1px solid #98bf21;
				padding:3px 7px 2px 7px;
				}
				#fields th 
				{
				font-size:1.4em;
				text-align:left;
				padding-top:5px;
				padding-bottom:4px;
				background-color:#A7C942;
				color:#fff;
				}
				#fields tr.alt td 
				{
				color:#000;
				background-color:#EAF2D3;
				}
				.codedvalues 
				{
				font-family:Arial, Helvetica, sans-serif;
				border-collapse:collapse;
				}
				.codedvalues td, .codedvalues th 
				{
				font-size:1.2em;
				border:1px solid #98bf21;
				padding:3px 7px 2px 7px;
				}
				.codedvalues th 
				{
				font-size:1.4em;
				text-align:left;
				padding-top:5px;
				padding-bottom:4px;
				background-color:#A7C942;
				color:#fff;
				}
				.codedvalues tr.alt td 
				{
				color:#000;
				background-color:#EAF2D3;
				}
				</style>
			</head>
			<body>
				<xsl:if test="/g:Geodatabase">
					<a name="_top">
					<h1>Geodatabase '<xsl:value-of select="/g:Geodatabase/@name"/>'</h1>
					</a>
					<p>Documentation of the Geodatabase Feature Classes and Object Classes that represent feature types:</p>
				</xsl:if>
				<xsl:for-each select="/g:Geodatabase/g:Type|/g:Type">
					<xsl:sort select="@name"/>
					<xsl:choose>
						<xsl:when test="@xlink:href">
							<xsl:apply-templates select="." mode="link"/>
						</xsl:when>
						<xsl:otherwise>
							<xsl:apply-templates select="." mode="inline"/>
						</xsl:otherwise>
					</xsl:choose>
				</xsl:for-each>
				<xsl:if test="/g:Geodatabase">
					<p> </p>
					<a name="namespaces">
					<p>Namespaces of the application schemas in the Geodatabase:</p>
					</a>
					<table class="codedvalues">
						<xsl:for-each select="/g:Geodatabase/g:Namespace">
						  <xsl:sort select="@prefix"/>
						  <tr>
								<td><p><nobr><xsl:value-of select="@prefix"/></nobr></p></td>
								<td><p><xsl:value-of select="@ns"/></p></td>
						  </tr>
						</xsl:for-each>
					</table>
				</xsl:if>
				
			</body>
		</html>
	</xsl:template>
	<xsl:template name="SubtypeTable">
		<p><b>Table: Feature types stored in <xsl:value-of select="@type"/> '<xsl:value-of select="@name"/>'</b></p>
		<table id="subtypes">
			<tr>
				<th>
					<p><b>Feature type</b></p>
				</th>
				<th>
					<p><b>Source application schema</b></p>
				</th>
				<th>
					<p><b>Subtype code</b></p>
				</th>
				<th>
					<p><b>Documentation</b></p>
				</th>
			</tr>
			<xsl:for-each select="g:Types/g:Subtype">
			  <xsl:sort select="@name"/>
				<xsl:apply-templates select="."/>
			</xsl:for-each>
		</table>
	</xsl:template>
	<xsl:template match="g:Subtype">
		<tr>
			<xsl:if test="position() mod 2 = 0">
        <xsl:attribute name="class">alt</xsl:attribute>
      </xsl:if>
      <td><p><nobr><xsl:value-of select="@name"/></nobr></p></td>
			<td><p><xsl:value-of select="@appSchema"/></p></td>
			<td><p><nobr><xsl:value-of select="@subtype"/></nobr></p></td>
			<td>
				<xsl:for-each select="g:Text">
					<p><xsl:value-of select="."/></p>
				</xsl:for-each>
			</td>
		</tr>
	</xsl:template>
	<xsl:template match="g:Field">
		<tr>
			<xsl:if test="position() mod 2 = 0">
        <xsl:attribute name="class">alt</xsl:attribute>
      </xsl:if>
			<td><p><nobr><xsl:value-of select="@name"/></nobr></p></td>
			<td><p><xsl:value-of select="@modelName"/></p></td>
			<td>
				<xsl:for-each select="g:SubtypeRule[count(@dependentDataElement)=0]">
					<xsl:sort select="."/>
					<p><xsl:value-of select="."/></p>
				</xsl:for-each>
				<xsl:if test="g:SubtypeRule[@dependentDataElement='true']">
					<p><b>Valid subtypes of <xsl:value-of select="../../@name"/>:</b></p>
					<xsl:for-each select="g:SubtypeRule[@dependentDataElement='true']">
						<xsl:sort select="."/>
						<p><xsl:value-of select="."/></p>
					</xsl:for-each>						
				</xsl:if>
			</td>
			<td><p><xsl:value-of select="substring-after(@type,'esriFieldType')"/><xsl:if test="@type='esriFieldTypeString'">(<xsl:value-of select="@length"/>)</xsl:if></p></td>
			<td>
				<xsl:for-each select="g:Text">
					<p><xsl:value-of select="."/></p>
				</xsl:for-each>
				<xsl:if test="g:CodedValue">
					<p>Valid values:</p>
					<table class="codedvalues">
						<xsl:for-each select="g:CodedValue">
						  <xsl:sort select="@code"/>
						  <tr>
								<td><p><xsl:value-of select="@code"/></p></td>
								<td>
									<xsl:for-each select="g:Text">
										<p><xsl:value-of select="."/></p>
									</xsl:for-each>
								</td>
						  </tr>
						</xsl:for-each>
					</table>
				</xsl:if>
			</td>
		</tr>
	</xsl:template>
	<xsl:template match="g:Type" mode="inline">
		<a><xsl:attribute name="name">_<xsl:value-of select="@name"/></xsl:attribute>
		<h2><xsl:value-of select="@type"/> '<xsl:value-of select="@name"/>'</h2>
		</a>
		<xsl:variable name="nam" select="@name"/>
		<h3>Index</h3>
		<ul>
			<li><a><xsl:attribute name="href">#_D</xsl:attribute>General Description</a></li>
			<li><a><xsl:attribute name="href">#_F</xsl:attribute>Fields</a></li>
			<xsl:for-each select="//g:DataElements/g:Type">
				<li><a><xsl:attribute name="href">#<xsl:value-of select="@name"/></xsl:attribute><xsl:value-of select="@name"/></a> (dependent <xsl:value-of select="@type"/>)</li>
			</xsl:for-each>
			<xsl:for-each select="//g:Relationship">
				<li><a><xsl:attribute name="href">#<xsl:value-of select="@name"/></xsl:attribute><xsl:value-of select="@name"/></a> (n:m relationship to 
				'<a><xsl:attribute name="href">Documentation_<xsl:value-of select="g:Target/@name"/>.xml</xsl:attribute><xsl:value-of select="g:Target/@name"/></a>')</li>
			</xsl:for-each>
		</ul>
		<a name="_D">
			<h3>General description</h3>
		</a>
		<xsl:for-each select="g:Text">
			<p><xsl:value-of select="."/></p>
		</xsl:for-each>
		<xsl:if test="g:Types/g:Subtype">
			<xsl:call-template name="SubtypeTable"/>
		</xsl:if>
		<p class="rightalign"><a><xsl:attribute name="href">#_<xsl:value-of select="$nam"/></xsl:attribute>back to top</a></p>
		<p class="rightalign"><a><xsl:attribute name="href">Documentation_index.xml</xsl:attribute>back to overview</a></p>
		<hr/>			
		<a name="_F">
			<h3>Fields</h3>
		</a>
		<xsl:choose>
			<xsl:when test="g:Fields/g:Field">
				<table id="fields">
					<tr>
						<th>
							<p><b>Field</b></p>
						</th>
						<th>
							<p><b>Property name</b></p>
						</th>
						<th>
							<p><b>In feature types</b></p>
						</th>
						<th>
							<p><b>Type</b></p>
						</th>
						<th>
							<p><b>Documentation</b></p>
						</th>
					</tr>
					<xsl:for-each select="g:Fields/g:Field">
					  <xsl:sort select="@name"/>
							<xsl:apply-templates select="."/>
					</xsl:for-each>
				</table>					
			</xsl:when>
			<xsl:otherwise>
				<p>No fields.</p>
			</xsl:otherwise>
		</xsl:choose>						
		<p class="rightalign"><a><xsl:attribute name="href">#_<xsl:value-of select="$nam"/></xsl:attribute>back to top</a></p>
		<p class="rightalign"><a><xsl:attribute name="href">Documentation_index.xml</xsl:attribute>back to overview</a></p>
		<hr/>			
		<xsl:for-each select="g:DataElements/g:Type|g:DataElements/g:Type/g:DataElements/g:Type|g:DataElements/g:Type/g:DataElements/g:Type/g:DataElements/g:Type">
			<a><xsl:attribute name="name"><xsl:value-of select="@name"/></xsl:attribute>
			<h3>Dependent <xsl:value-of select="@type"/> '<xsl:value-of select="@name"/>'</h3>
			</a>
			<h4>General description</h4>
			<xsl:for-each select="g:Text">
				<p><xsl:value-of select="."/></p>
			</xsl:for-each>
			<xsl:if test="g:Types/g:Subtype">
				<xsl:call-template name="SubtypeTable"/>
			</xsl:if>
			<h4>Fields</h4>
			<xsl:choose>
				<xsl:when test="g:Fields/g:Field">
				<table id="fields">
					<tr>
						<th>
							<p><b>Field</b></p>
						</th>
						<th>
							<p><b>Property name</b></p>
						</th>
						<th>
							<p><b>In feature types</b></p>
						</th>
						<th>
							<p><b>Type</b></p>
						</th>
						<th>
							<p><b>Documentation</b></p>
						</th>
					</tr>
					<xsl:for-each select="g:Fields/g:Field">
					  <xsl:sort select="@name"/>
							<xsl:apply-templates select="."/>
					</xsl:for-each>
				</table>					
				</xsl:when>
				<xsl:otherwise>
					<p>No fields.</p>
				</xsl:otherwise>
			</xsl:choose>			
			<p class="rightalign"><a><xsl:attribute name="href">#_<xsl:value-of select="$nam"/></xsl:attribute>back to top</a></p>
			<p class="rightalign"><a><xsl:attribute name="href">Documentation_index.xml</xsl:attribute>back to overview</a></p>
			<hr/>			
		</xsl:for-each>
		<xsl:variable name="tab" select="@name"/>
		<xsl:for-each select="//g:Relationship">
			<a><xsl:attribute name="name"><xsl:value-of select="@name"/></xsl:attribute>
			<h3>Relationship table '<xsl:value-of select="@name"/>'</h3>
			</a>
			<h4>General description</h4>
			<xsl:for-each select="g:Text">
				<p><xsl:value-of select="."/></p>
			</xsl:for-each>
			<h4>Fields</h4>
			<xsl:choose>
				<xsl:when test="g:Field">
				<table id="fields">
					<tr>
						<th>
							<p><b>Field</b></p>
						</th>
						<th>
							<p><b>Property name</b></p>
						</th>
						<th>
							<p><b>In feature types</b></p>
						</th>
						<th>
							<p><b>Type</b></p>
						</th>
						<th>
							<p><b>Documentation</b></p>
						</th>
					</tr>
					<xsl:for-each select="g:Field">
					  <xsl:sort select="@name"/>
							<xsl:apply-templates select="."/>
					</xsl:for-each>
				</table>					
				</xsl:when>
				<xsl:otherwise>
					<p>No fields.</p>
				</xsl:otherwise>
			</xsl:choose>						
			<p class="rightalign"><a><xsl:attribute name="href">#_<xsl:value-of select="$nam"/></xsl:attribute>back to top</a></p>
			<p class="rightalign"><a><xsl:attribute name="href">Documentation_index.xml</xsl:attribute>back to overview</a></p>
			<hr/>			
		</xsl:for-each>
	</xsl:template>
	<xsl:template match="g:Type" mode="link">
		<ul>
			<xsl:variable name="nam" select="@name"/>
			<li><a><xsl:attribute name="href">Documentation_<xsl:value-of select="$nam"/>.xml</xsl:attribute><xsl:value-of select="$nam"/></a>  (<xsl:value-of select="@type"/>)</li>
			<ul>
			<xsl:for-each select="g:DataElements/g:Type">
				<li><a><xsl:attribute name="href">Documentation_<xsl:value-of select="$nam"/>.xml#<xsl:value-of select="@name"/></xsl:attribute><xsl:value-of select="@name"/></a>  (<xsl:value-of select="@type"/>)</li>
			</xsl:for-each>
			<xsl:for-each select="g:DataElements/g:Type">
				<xsl:variable name="namDE" select="@name"/>
				<xsl:for-each select="/g:Geodatabase/g:Relationship[g:Source/@name=$namDE]">
					<li><a><xsl:attribute name="href">Documentation_<xsl:value-of select="$nam"/>.xml#<xsl:value-of select="@name"/></xsl:attribute><xsl:value-of select="@name"/></a> (n:m relationship to '<a><xsl:attribute name="href">Documentation_<xsl:value-of select="g:Target/@name"/>.xml</xsl:attribute><xsl:value-of select="g:Target/@name"/></a>')</li>
				</xsl:for-each>
			</xsl:for-each>
			<xsl:for-each select="/g:Geodatabase/g:Relationship[g:Source/@name=$nam]">
				<li><a><xsl:attribute name="href">Documentation_<xsl:value-of select="$nam"/>.xml#<xsl:value-of select="@name"/></xsl:attribute><xsl:value-of select="@name"/></a> (n:m relationship to '<a><xsl:attribute name="href">Documentation_<xsl:value-of select="g:Target/@name"/>.xml</xsl:attribute><xsl:value-of select="g:Target/@name"/></a>')</li>
			</xsl:for-each>
			<xsl:for-each select="/g:Geodatabase/g:Relationship[g:Target/@name=$nam]">
				<xsl:variable name="namR" select="@name"/>
				<xsl:choose>
					<xsl:when test="/g:Geodatabase/g:Type/@name=$namR">
						<li><a><xsl:attribute name="href">Documentation_<xsl:value-of select="g:Source/@name"/>.xml#<xsl:value-of select="@name"/></xsl:attribute><xsl:value-of select="@name"/></a> (n:m relationship from '<a><xsl:attribute name="href">Documentation_<xsl:value-of select="g:Source/@name"/>.xml</xsl:attribute><xsl:value-of select="g:Source/@name"/></a>')</li>
					</xsl:when>
					<xsl:when test="/g:Geodatabase/g:Type/g:DataElements/g:Type/@name=$namR">
						<xsl:variable name="ft" select="/g:Geodatabase/g:Type[g:DataElements/g:Type/@name=$namR]"/>
						<li><a><xsl:attribute name="href">Documentation_<xsl:value-of select="$ft"/>.xml#<xsl:value-of select="@name"/></xsl:attribute><xsl:value-of select="@name"/></a> (n:m relationship of a dependent type from '<a><xsl:attribute name="href">Documentation_<xsl:value-of select="$ft"/>.xml</xsl:attribute><xsl:value-of select="$ft"/></a>')</li>
					</xsl:when>
				</xsl:choose>
			</xsl:for-each>
			</ul>
		</ul>
	</xsl:template>
</xsl:stylesheet>