### AlterFields.py ###
# Alter fields
# Excel Columns Expected to be in the following order:
# Feature Class/Table    Field Name  Field Type  Field Length(Optional) Field Is Nullable(Optional) New Field Name(Optional) New Alias(Optional)
# <FC or Table Name>     <fieldName> TEXT         20
#
# SDE path must be full path to .sde connection file.

import arcpy
import sys
from xlrd import open_workbook
import logging
import time
import os


def AlterField(inTable, fieldName, fieldType, fieldLength, fieldIsNullable, newFieldName, newAlias, field_precision, field_scale, fieldDomain):

    #Set local variables
    new_field_name = fieldName
    if newFieldName:
        new_field_name = newFieldName
    new_field_alias = newAlias
    new_type = fieldType
    if fieldLength:
        new_length = int(fieldLength)
    new_is_nullable = fieldIsNullable
    clear_field_alias = "FALSE"    
    field_name = fieldName #short int, non nullable field    

    # Check if field exists in fc/table 
    field_names = [f.name for f in arcpy.ListFields(inTable)]
    #if fieldName in field_names:
    if field_name.lower() not in [x.lower() for x in field_names]:
        msg = 'Field ' + fieldName + ' does not exist in FeatureClass/Table ' + inTable
        logging.info(msg)
        print msg
        return

    # If new_length or new_is_nullable then we need to delete and add new field
    if new_length or new_is_nullable:
        # We need to copy the records in a temporary field, 
        # delete the original field, add new field with changes, and copy the records back.
        msg = "Checking if field, {0}, has records.".format(field_name)
        logging.info(msg)
        print msg
        has_records = False
        with arcpy.da.SearchCursor(inTable, field_name) as cursor:        
            for row in cursor:#only includes rows where at least one of the three are empty
                if not row[0] in [None]:
                    has_records = True                    
                    break
        # if has_records, then create temporary field
        if has_records:
            temp_field_name = field_name + "_temp"
            msg = "Field, {0}, has records. Getting settings for temporary backup field.".format(field_name)
            logging.info(msg)
            print msg
            orig_field = arcpy.ListFields(inTable, field_name)
            if orig_field:
                orig_field_type = orig_field[0].type
                orig_field_scale = orig_field[0].scale
                orig_field_precision = orig_field[0].precision
                orig_field_length = orig_field[0].length               
            
                #addField(fcPath, temp_field_name, orig_field_type, orig_field_precision, orig_field_scale, orig_field_length)
                msg = "Adding temorary field {0} to {1}.".format(temp_field_name, inTable)
                logging.info(msg)
                print msg
                arcpy.AddField_management(inTable, temp_field_name, orig_field_type, orig_field_precision, orig_field_scale, new_length)
                msg = "Running CalculateField from {0} to {1}".format(field_name, temp_field_name)
                logging.info(msg)
                print msg
                arcpy.CalculateField_management(inTable, temp_field_name, "!" + field_name + "!", "PYTHON")


                # Verify field copy
                msg = "Verifying CalcualteField from {0} to {1}".format(field_name, temp_field_name)
                logging.info(msg)
                print msg
                copy_okay = True
                msg = ""
                orig_values = []
                with arcpy.da.SearchCursor(inTable, field_name) as orig_cursor:
                    for row in orig_cursor:
                        orig_values.append(row[0])
                with arcpy.da.SearchCursor(inTable, temp_field_name) as temp_cursor:
                    i = 0
                    for row in temp_cursor:
                        if row[0] != orig_values[i]:
                            copy_okay = False
                            msg = "Original Value: {0}  does not equal Temp Value: {1}  for row: {2}  Field {3} will not be updated.".format(row[0], orig_values[i], i, field_name)
                        i = i + 1
                if not copy_okay:
                    logging.error(msg)
                    print (msg)
                    return

        #Delete field
        # Try to Drop Field
        try:
            arcpy.DeleteField_management(inTable, field_name)
            msg = "Dropped Field: " + fieldName + ' from ' + inTable
            logging.info(msg)
            print msg
        except Exception as err:
            msg = 'Error running AddField_management! ' + err.args[0] + '  FeatureClass/Table:' + inTable + '  Field:' + fieldName
            logging.error(msg)
            print msg
            return

        #Add field
        # Try to Add Field
        try:
            # Setup parameters
            fieldIsNull = None
            if new_is_nullable:
              if new_is_nullable.lower() == 'no':
                fieldIsNull = 'NON_NULLABLE'
            fieldPrec = None
            if field_precision:
              fieldPrec = long(field_precision)
            fieldScl = None
            if field_scale:
              fieldScl = long(field_scale)
            fieldLen = None
            if new_length:
              fieldLen = long(new_length)
            arcpy.AddField_management(inTable, new_field_name, new_type, fieldPrec, fieldScl, fieldLen, new_field_alias, fieldIsNull,field_domain=fieldDomain)
            msg = "Added Field: " + fieldName + ' to ' + inTable
            logging.info(msg)
            print msg
        except Exception as err:
            msg = 'Error running AddField_management! ' + err.args[0] + '  FeatureClass/Table:' + inTable + '  Field:' + fieldName
            logging.error(msg)
            print msg
            return

        # Copy data values back to new field
        msg = "Running CalcualteField from {0} to {1}".format(temp_field_name, new_field_name)
        logging.info(msg)
        print msg
        arcpy.CalculateField_management(inTable, new_field_name, "!" + temp_field_name + "!", "PYTHON")

        # Verify records
        # Verify field copy
        msg = "Verifying CalcualteField from {0} to {1}".format(temp_field_name, new_field_name)
        logging.info(msg)
        print msg
        copy_okay = True
        msg = ""
        orig_values = []
        with arcpy.da.SearchCursor(inTable, field_name) as orig_cursor:
            for row in orig_cursor:
                orig_values.append(row[0])
        with arcpy.da.SearchCursor(inTable, temp_field_name) as temp_cursor:
            i = 0
            for row in temp_cursor:
                if row[0] != orig_values[i]:
                    copy_okay = False
                    msg = "Original Value: {0}  does not equal Temp Value: {1}  for row: {2}  Field {3} will not be updated.".format(row[0], orig_values[i], i, field_name)
                i = i + 1
        if not copy_okay:
            logging.error(msg)
            print (msg)
            return

        #Delete temporary field
        # Try to Drop Field
        try:
            arcpy.DeleteField_management(inTable, temp_field_name)
            msg = "Dropped Field: " + temp_field_name + ' from ' + inTable
            logging.info(msg)
            print msg
        except Exception as err:
            msg = 'Error running AddField_management! ' + err.args[0] + '  FeatureClass/Table:' + inTable + '  Field:' + fieldName
            logging.error(msg)
            print msg
            return
    else:
        #Alter the properties of a non nullable, short data type field to become a text field
        msg = "Altering Field: " + field_name
        logging.info(msg)
        print msg
        arcpy.AlterField_management(inTable, field_name, new_field_name, new_field_alias, new_type, new_length, new_is_nullable, clear_alias)


def main(argv):   
    # Setup Logging
    timestr = time.strftime("%Y%m%d-%H%M%S")
    logFileName = 'AlterFields_' + timestr + '.log'
    logging.basicConfig(filename=logFileName, level=logging.INFO, format='%(asctime)s %(message)s')
    logging.info('Started AlterFields.py')
    print 'Started AlterFields.py'
    
    excel_doc = None
    wrkspc = None

    if len(argv) > 1:
      excel_doc = argv[1]
    if len(argv) > 2:
      sde_conn = argv[2]

    try:
        if not excel_doc:
            excel_doc = raw_input("Please provide path to Excel doc: ")

        if not sde_conn:
            sde_conn = raw_input("Please provide path to SDE connection file: ")

        if not os.path.isabs(excel_doc):
            wkdir = os.path.dirname(os.path.realpath(__file__))
            excel_doc = os.path.join(wkdir, excel_doc)
        
        #Verify workspace exists
        if not arcpy.Exists(sde_conn):
            msg = "Error: Workspace {0} does not exist!".format(sde_conn)
            logging.error(msg)
            print msg
            sys.exit()

        #Verify Excel doc exists
        if not os.path.exists(excel_doc):
            msg = "Error: Excel document {} does not exist!".format(excel_doc)
            logging.error(msg)
            print msg
            sys.exit() 
        
        logging.info('Excel doc: ' + excel_doc)
        print 'Excel doc: ' + excel_doc

        # Read Excel Sheet
        wb = open_workbook(excel_doc)
        for sheet in wb.sheets():
            # Only use sheet 'AlterFields'
            if sheet.name != 'AlterFields':
                continue
            number_of_rows = sheet.nrows
            number_of_columns = sheet.ncols
            logging.info(' ')
            print ' '
            logging.info('Processing Excel sheet: ' + sheet.name)
            print 'Processing Excel sheet: ' + sheet.name
            if number_of_rows > 1:
                print "Rows: " + str(number_of_rows)
          
                # Read column values from row
                for row in range(1, number_of_rows):
                    in_table = (sheet.cell(row,0).value)
                    field_name = (sheet.cell(row,1).value)
                    field_type = (sheet.cell(row,2).value)
                    field_length = (sheet.cell(row,3).value)
                    field_is_nullable = (sheet.cell(row,4).value)
                    new_field_name = (sheet.cell(row,5).value)
                    new_field_alias = (sheet.cell(row,6).value)
                    new_field_precision = (sheet.cell(row,7).value)
                    new_field_scale = (sheet.cell(row,8).value)
                    fieldDomain = (sheet.cell(row,9).value)

                    # Put together full sde path.
                    in_table = os.path.join(sde_conn, in_table)

                    # Check if db object exists
                    if not arcpy.Exists(in_table):
                        msg = "Skipping " + in_table + "! Data object does not exist."
                        logging.info(msg)
                        print msg  
                        continue

                    # Check db object type from path
                    desc = arcpy.Describe(in_table)
                    if hasattr(desc, "dataType"):
                        print("DataType:    " + desc.dataType)
                    else:
                        msg = "Skipping " + in_table + "! Data object does not have a dataType attribute."
                        logging.info(msg)
                        print msg  
              
                    arcpy.env.workspace = in_table
                    logging.info('Workspace: ' + in_table)

                    # Check type of workspace and handle appropriately
                    fcTable = ['FeatureClass', 'Table']
                    if desc.dataType in fcTable:
                        msg = 'PROCESSING FC/TABLE: ' + in_table
                        logging.info(msg)
                        print msg
                        msg = 'Processing field: ' + field_name
                        logging.info(msg)
                        print msg 

                        AlterField(in_table, field_name, field_type, field_length, field_is_nullable, 
                                   new_field_name, new_field_alias, new_field_precision, new_field_scale, fieldDomain)

                    else:                
                        msg = "{0} is not a recognized datatype.".format(desc.dataType)
                        logging.info(msg)
                        print msg
                        continue             
            else:
                logging.warning('No rows found in excel doc!')
                print ('No rows found inexcel doc!')

        logging.info('Finished')
        print 'Finished'
 
    except Exception as err:
        import traceback, sys
        tb = sys.exc_info()[2]
        logging.error("Line %i" % tb.tb_lineno + "  " + err.message)
        print "Line %i" % tb.tb_lineno + "  " + err.message
        


if __name__ == '__main__':
    main(sys.argv) 