### RelationshipOperations.py ###
# Perfoms relationship operations 
# Excel Columns Expected to be in the following order:
#  Table Name	      Operation	  
#  <Table Name>       <Create or Delete>
#   
# SDE path must be full path to .sde connection file
# Excel doc must be full path.
# Usage: python.exe <Path>\AddFields.py "<Path>\MyExcelDoc.xls"  "<Path>\MySDEConn.sde"

import arcpy
import sys
from xlrd import open_workbook
import logging
import time
import os

# Create table function
def CreateRelationship (workspace,row):  
    destination_primary_key = None
    destination_foreign_key = None

    # Try to create relationship
    try:
        # Get values from row
        origin_table = (sheet.cell(row,0).value)
        destination_table = (sheet.cell(row,1).value)
        out_relationship_class = (sheet.cell(row,2).value)
        relationship_type = (sheet.cell(row,3).value)
        forward_label = (sheet.cell(row,4).value)
        backward_label = (sheet.cell(row,5).value)
        message_direction = (sheet.cell(row,6).value)
        cardinality = (sheet.cell(row,7).value)
        attributed = (sheet.cell(row,8).value) 
        origin_primary_key = (sheet.cell(row,9).value)
        origin_foreign_key = (sheet.cell(row,10).value)
        if sheet.cell(row,11).value:
            destination_primary_key = (sheet.cell(row,11).value)
        if sheet.cell(row,12).value:
            destination_foreign_key = (sheet.cell(row,12).value)
        
        arcpy.env.workspace = workspace
        
        #Verify tables exist
        if not arcpy.Exists(origin_table):
            msg = "Skipping {0}! Data object {1} exists.".format(out_relationship_class, origin_table)
            logging.info(msg)
            print msg
            return

        if not arcpy.Exists(destination_table):
            msg = "Skipping {0}! Data object {1} exists.".format(out_relationship_class, destination_table)
            logging.info(msg)
            print msg
            return

        arcpy.CreateRelationshipClass_management(origin_table, destination_table, out_relationship_class, relationship_type,
                                         forward_label, backward_label, message_direction, cardinality, attributed, 
                                         origin_primary_key, origin_foreign_key, destination_primary_key, destination_foreign_key)

        msg = "Relationship class  {0}  created!".format(out_relationship_class)
        logging.info(msg)
        print msg
        
    except Exception as err:
        msg = 'Error running CreateTable_management! ' + err.args[0] + '  Table:' + table_name
        logging.error(msg)
        print msg
    


excel_doc = arcpy.GetParameterAsText(0)
sde_conn = arcpy.GetParameterAsText(1)

try:
    # Set the parameters here in the script if not passing in through command
    # line arguments.
    #sde_conn = r"C:\Users\Myprofile\AppData\Roaming\ESRI\Desktop10.5\ArcCatalog\rblischke.ag4i.sde"
    #excel_doc = r"C:\Users\Myprofile\Documents\EMCS\AddFields.xlsx"
    if not excel_doc:
            excel_doc = raw_input("Please provide path to Excel doc: ")

    if not sde_conn:
        sde_conn = raw_input("Please provide path to SDE connection file: ")

    # Setup Logging
    # Check excel_doc and try parse out theme name
    if excel_doc:
        excel_filename = os.path.basename(excel_doc)
        # Get string before the underscore
        theme_name = excel_filename.split('_')[0]
    timestr = time.strftime("%Y%m%d-%H%M%S")
    logFileName = 'RelationshipOperations_' + theme_name + '_' + timestr + '.log'
    logging.basicConfig(filename=logFileName, level=logging.INFO, format='%(asctime)s %(message)s')
    logging.info('Started')
    print 'Started'
    
    if not os.path.isabs(excel_doc):
        wkdir = os.path.dirname(os.path.realpath(__file__))
        excel_doc = os.path.join(wkdir, excel_doc)
        
    #Verify workspace exists
    if not arcpy.Exists(sde_conn):
        msg = "Error: Workspace {0} does not exist!".format(sde_conn)
        logging.error(msg)
        print msg
        sys.exit()

    #Verify Excel doc exists
    if not os.path.exists(excel_doc):
        msg = "Error: Excel document {} does not exist!".format(excel_doc)
        logging.error(msg)
        print msg
        sys.exit() 
        
    logging.info('Excel doc: ' + excel_doc)
    print 'Excel doc: ' + excel_doc

    # Read Excel Sheet
    wb = open_workbook(excel_doc)
    for sheet in wb.sheets():
        # Only use sheet 'AddFields'
        if sheet.name != 'RelationshipOperations':
            continue
        number_of_rows = sheet.nrows
        number_of_columns = sheet.ncols
        logging.info(' ')
        print ' '
        logging.info('Processing Excel sheet: ' + sheet.name)
        print 'Processing Excel sheet: ' + sheet.name
        if number_of_rows > 1:
            print "Rows: " + str(number_of_rows)
          
            create_operation = "Create"
            delete_operation = "Delete"

            # Read column values from row
            for row in range(1, number_of_rows):
                operation = (sheet.cell(row,13).value)
                            
                
                if operation == create_operation:
                    CreateRelationship(sde_conn, row)
                        
        else:
          logging.warning('No rows found in excel doc!')

    logging.info('Finished')
    print 'Finished'
 
except Exception as err:
    import traceback, sys
    tb = sys.exc_info()[2]
    logging.error("Line %i" % tb.tb_lineno)
    print "Line %i" % tb.tb_lineno
    logging.error(err.message)
    print err.message
    

