### TableOperations.py ###
# Perfoms table operations 
# Excel Columns Expected to be in the following order:
#  Table Name	      Operation	  
#  <Table Name>       <Create or Delete>
#   
# SDE path must be full path to .sde connection file
# Excel doc must be full path.
# Usage: python.exe <Path>\AddFields.py "<Path>\MyExcelDoc.xls"  "<Path>\MySDEConn.sde"

import arcpy
import sys
from xlrd import open_workbook
import logging
import time
import os

# Create table function
def createTable (workspace,table_name):  
    # Try to create table
    try:
        arcpy.CreateTable_management(workspace, table_name)
        msg = "Table {0} created!".format(table_name)
        logging.info(msg)
        print msg
        
    except Exception as err:
        msg = 'Error running CreateTable_management! ' + err.args[0] + '  Table:' + table_name
        logging.error(msg)
        print msg
    
  
excel_doc = arcpy.GetParameterAsText(0)
sde_conn = arcpy.GetParameterAsText(1)

try:
    # Set the parameters here in the script if not passing in through command
    # line arguments.
    #sde_conn = r"C:\Users\Myprofile\AppData\Roaming\ESRI\Desktop10.5\ArcCatalog\rblischke.ag4i.sde"
    #excel_doc = r"C:\Users\Myprofile\Documents\EMCS\AddFields.xlsx"
    if not excel_doc:
            excel_doc = raw_input("Please provide path to Excel doc: ")

    if not sde_conn:
        sde_conn = raw_input("Please provide path to SDE connection file: ")
    
    # Setup Logging
    # Check excel_doc and try parse out theme name
    if excel_doc:
        excel_filename = os.path.basename(excel_doc)
        # Get string before the underscore
        theme_name = excel_filename.split('_')[0]
    timestr = time.strftime("%Y%m%d-%H%M%S")
    logFileName = 'TableOperations_' + theme_name + '_' + timestr + '.log'
    logging.basicConfig(filename=logFileName, level=logging.INFO, format='%(asctime)s %(message)s')
    logging.info('Started')
    print 'Started'

    if not os.path.isabs(excel_doc):
        wkdir = os.path.dirname(os.path.realpath(__file__))
        excel_doc = os.path.join(wkdir, excel_doc)
        
    #Verify workspace exists
    if not arcpy.Exists(sde_conn):
        msg = "Error: Workspace {0} does not exist!".format(sde_conn)
        logging.error(msg)
        print msg
        sys.exit()

    #Verify Excel doc exists
    if not os.path.exists(excel_doc):
        msg = "Error: Excel document {} does not exist!".format(excel_doc)
        logging.error(msg)
        print msg
        sys.exit() 
        
    logging.info('Excel doc: ' + excel_doc)
    print 'Excel doc: ' + excel_doc

    # Read Excel Sheet
    wb = open_workbook(excel_doc)
    for sheet in wb.sheets():
        # Only use sheet 'AddFields'
        if sheet.name != 'TableOperations':
            continue
        number_of_rows = sheet.nrows
        number_of_columns = sheet.ncols
        logging.info(' ')
        print ' '
        logging.info('Processing Excel sheet: ' + sheet.name)
        print 'Processing Excel sheet: ' + sheet.name
        if number_of_rows > 1:
            print "Rows: " + str(number_of_rows)
          
            create_operation = "Create"
            delete_operation = "Delete"

            # Read column values from row
            for row in range(1, number_of_rows):
                table_name = (sheet.cell(row,0).value)
                operation = (sheet.cell(row,1).value)
                            
                # Put together full sde path.
                in_table = os.path.join(sde_conn, table_name)

                # Check if db object exists
                if arcpy.Exists(in_table):
                    msg = "Skipping " + table_name + "! Data object exists."
                    logging.info(msg)
                    print msg  
                    continue
              
                msg = 'Processing table: ' + table_name
                logging.info(msg)
                print msg 
                if operation == create_operation:
                    createTable(sde_conn, table_name)
                        
        else:
          logging.warning('No rows found in excel doc!')

    logging.info('Finished')
    print 'Finished'
 
except Exception as err:
    import traceback, sys
    tb = sys.exc_info()[2]
    logging.error("Line %i" % tb.tb_lineno)
    print "Line %i" % tb.tb_lineno
    logging.error(err.message)
    print err.message
    
